@@ -1,3 +1,4 @@
+import re
 from collections import defaultdict
 from datetime import datetime
 from typing import Iterable, Iterator, Optional, Union, cast
@@ -18,18 +19,13 @@ from phoenix.config import (
     get_env_database_allocated_storage_capacity_gibibytes,
     getenv,
 )
-from phoenix.db import enums, models
-from phoenix.db.helpers import SupportedSQLDialect
-from phoenix.db.models import DatasetExample as OrmExample
-from phoenix.db.models import DatasetExampleRevision as OrmRevision
-from phoenix.db.models import DatasetVersion as OrmVersion
-from phoenix.db.models import Experiment as OrmExperiment
-from phoenix.db.models import ExperimentRun as OrmExperimentRun
-from phoenix.db.models import Trace as OrmTrace
+from phoenix.db import models
+from phoenix.db.constants import DEFAULT_PROJECT_TRACE_RETENTION_POLICY_ID
+from phoenix.db.helpers import SupportedSQLDialect, exclude_experiment_projects
 from phoenix.pointcloud.clustering import Hdbscan
 from phoenix.server.api.auth import MSG_ADMIN_ONLY, IsAdmin
 from phoenix.server.api.context import Context
-from phoenix.server.api.exceptions import NotFound, Unauthorized
+from phoenix.server.api.exceptions import BadRequest, NotFound, Unauthorized
 from phoenix.server.api.helpers import ensure_list
 from phoenix.server.api.helpers.experiment_run_filters import (
     ExperimentRunFilterConditionSyntaxError,
@@ -40,9 +36,12 @@ from phoenix.server.api.helpers.playground_clients import initialize_playground_
 from phoenix.server.api.helpers.playground_registry import PLAYGROUND_CLIENT_REGISTRY
 from phoenix.server.api.input_types.ClusterInput import ClusterInput
 from phoenix.server.api.input_types.Coordinates import InputCoordinate2D, InputCoordinate3D
+from phoenix.server.api.input_types.DatasetFilter import DatasetFilter
 from phoenix.server.api.input_types.DatasetSort import DatasetSort
 from phoenix.server.api.input_types.InvocationParameters import InvocationParameter
-from phoenix.server.api.subscriptions import PLAYGROUND_PROJECT_NAME
+from phoenix.server.api.input_types.ProjectFilter import ProjectFilter
+from phoenix.server.api.input_types.ProjectSort import ProjectColumn, ProjectSort
+from phoenix.server.api.types.AnnotationConfig import AnnotationConfig, to_gql_annotation_config
 from phoenix.server.api.types.Cluster import Cluster, to_gql_clusters
 from phoenix.server.api.types.Dataset import Dataset, to_gql_dataset
 from phoenix.server.api.types.DatasetExample import DatasetExample
@@ -58,21 +57,32 @@ from phoenix.server.api.types.Experiment import Experiment
 from phoenix.server.api.types.ExperimentComparison import ExperimentComparison, RunComparisonItem
 from phoenix.server.api.types.ExperimentRun import ExperimentRun, to_gql_experiment_run
 from phoenix.server.api.types.Functionality import Functionality
-from phoenix.server.api.types.GenerativeModel import GenerativeModel
+from phoenix.server.api.types.GenerativeModel import GenerativeModel, to_gql_generative_model
 from phoenix.server.api.types.GenerativeProvider import GenerativeProvider, GenerativeProviderKey
+from phoenix.server.api.types.InferenceModel import InferenceModel
 from phoenix.server.api.types.InferencesRole import AncillaryInferencesRole, InferencesRole
-from phoenix.server.api.types.Model import Model
 from phoenix.server.api.types.node import from_global_id, from_global_id_with_expected_type
-from phoenix.server.api.types.pagination import ConnectionArgs, CursorString, connection_from_list
+from phoenix.server.api.types.pagination import (
+    ConnectionArgs,
+    Cursor,
+    CursorString,
+    connection_from_cursors_and_nodes,
+    connection_from_list,
+)
+from phoenix.server.api.types.PlaygroundModel import PlaygroundModel
 from phoenix.server.api.types.Project import Project
 from phoenix.server.api.types.ProjectSession import ProjectSession, to_gql_project_session
+from phoenix.server.api.types.ProjectTraceRetentionPolicy import ProjectTraceRetentionPolicy
 from phoenix.server.api.types.Prompt import Prompt, to_gql_prompt_from_orm
 from phoenix.server.api.types.PromptLabel import PromptLabel, to_gql_prompt_label
 from phoenix.server.api.types.PromptVersion import PromptVersion, to_gql_prompt_version
+from phoenix.server.api.types.PromptVersionTag import PromptVersionTag, to_gql_prompt_version_tag
 from phoenix.server.api.types.SortDir import SortDir
 from phoenix.server.api.types.Span import Span
+from phoenix.server.api.types.SpanAnnotation import SpanAnnotation, to_gql_span_annotation
 from phoenix.server.api.types.SystemApiKey import SystemApiKey
 from phoenix.server.api.types.Trace import Trace
+from phoenix.server.api.types.TraceAnnotation import TraceAnnotation, to_gql_trace_annotation
 from phoenix.server.api.types.User import User, to_gql_user
 from phoenix.server.api.types.UserApiKey import UserApiKey, to_gql_api_key
 from phoenix.server.api.types.UserRole import UserRole
@@ -107,20 +117,39 @@ class Query:
         ]
 
     @strawberry.field
-    async def models(self, input: Optional[ModelsInput] = None) -> list[GenerativeModel]:
+    async def generative_models(
+        self,
+        info: Info[Context, None],
+    ) -> list[GenerativeModel]:
+        async with info.context.db() as session:
+            result = await session.scalars(
+                select(models.GenerativeModel)
+                .where(models.GenerativeModel.deleted_at.is_(None))
+                .order_by(
+                    models.GenerativeModel.is_built_in.asc(),  # display custom models first
+                    models.GenerativeModel.provider.nullslast(),
+                    models.GenerativeModel.name,
+                )
+                .options(joinedload(models.GenerativeModel.token_prices))
+            )
+
+        return [to_gql_generative_model(model) for model in result.unique()]
+
+    @strawberry.field
+    async def playground_models(self, input: Optional[ModelsInput] = None) -> list[PlaygroundModel]:
         if input is not None and input.provider_key is not None:
             supported_model_names = PLAYGROUND_CLIENT_REGISTRY.list_models(input.provider_key)
             supported_models = [
-                GenerativeModel(name=model_name, provider_key=input.provider_key)
+                PlaygroundModel(name=model_name, provider_key=input.provider_key)
                 for model_name in supported_model_names
             ]
             return supported_models
 
         registered_models = PLAYGROUND_CLIENT_REGISTRY.list_all_models()
-        all_models: list[GenerativeModel] = []
+        all_models: list[PlaygroundModel] = []
         for provider_key, model_name in registered_models:
             if model_name is not None and provider_key is not None:
-                all_models.append(GenerativeModel(name=model_name, provider_key=provider_key))
+                all_models.append(PlaygroundModel(name=model_name, provider_key=provider_key))
         return all_models
 
     @strawberry.field
@@ -158,7 +187,7 @@ class Query:
         stmt = (
             select(models.User)
             .join(models.UserRole)
-            .where(models.UserRole.name != enums.UserRole.SYSTEM.value)
+            .where(models.UserRole.name != "SYSTEM")
             .order_by(models.User.email)
             .options(joinedload(models.User.role))
         )
@@ -174,7 +203,7 @@ class Query:
     ) -> list[UserRole]:
         async with info.context.db() as session:
             roles = await session.scalars(
-                select(models.UserRole).where(models.UserRole.name != enums.UserRole.SYSTEM.value)
+                select(models.UserRole).where(models.UserRole.name != "SYSTEM")
             )
         return [
             UserRole(
@@ -190,7 +219,7 @@ class Query:
             select(models.ApiKey)
             .join(models.User)
             .join(models.UserRole)
-            .where(models.UserRole.name != enums.UserRole.SYSTEM.value)
+            .where(models.UserRole.name != "SYSTEM")
         )
         async with info.context.db() as session:
             api_keys = await session.scalars(stmt)
@@ -202,7 +231,7 @@ class Query:
             select(models.ApiKey)
             .join(models.User)
             .join(models.UserRole)
-            .where(models.UserRole.name == enums.UserRole.SYSTEM.value)
+            .where(models.UserRole.name == "SYSTEM")
         )
         async with info.context.db() as session:
             api_keys = await session.scalars(stmt)
@@ -225,6 +254,8 @@ class Query:
         last: Optional[int] = UNSET,
         after: Optional[CursorString] = UNSET,
         before: Optional[CursorString] = UNSET,
+        sort: Optional[ProjectSort] = UNSET,
+        filter: Optional[ProjectFilter] = UNSET,
     ) -> Connection[Project]:
         args = ConnectionArgs(
             first=first,
@@ -232,18 +263,26 @@ class Query:
             last=last,
             before=before if isinstance(before, CursorString) else None,
         )
-        stmt = (
-            select(models.Project)
-            .outerjoin(
-                models.Experiment,
-                and_(
-                    models.Project.name == models.Experiment.project_name,
-                    models.Experiment.project_name != PLAYGROUND_PROJECT_NAME,
-                ),
+        stmt = select(models.Project)
+
+        if sort and sort.col is ProjectColumn.endTime:
+            # For end time sorting, we need to use a correlated subquery
+            # The end_time comes from the Trace model, and we need to get the max end_time for
+            # each project
+            end_time_subq = (
+                select(func.max(models.Trace.end_time))
+                .where(models.Trace.project_rowid == models.Project.id)
+                .scalar_subquery()
             )
-            .where(models.Experiment.project_name.is_(None))
-            .order_by(models.Project.id)
-        )
+            stmt = stmt.order_by(
+                end_time_subq.desc() if sort.dir is SortDir.desc else end_time_subq.asc()
+            )
+        elif sort:
+            sort_col = getattr(models.Project, sort.col.value)
+            stmt = stmt.order_by(sort_col.desc() if sort.dir is SortDir.desc else sort_col.asc())
+        if filter:
+            stmt = stmt.where(getattr(models.Project, filter.col.value).ilike(f"%{filter.value}%"))
+        stmt = exclude_experiment_projects(stmt)
         async with info.context.db() as session:
             projects = await session.stream_scalars(stmt)
             data = [
@@ -268,6 +307,7 @@ class Query:
         after: Optional[CursorString] = UNSET,
         before: Optional[CursorString] = UNSET,
         sort: Optional[DatasetSort] = UNSET,
+        filter: Optional[DatasetFilter] = UNSET,
     ) -> Connection[Dataset]:
         args = ConnectionArgs(
             first=first,
@@ -279,6 +319,8 @@ class Query:
         if sort:
             sort_col = getattr(models.Dataset, sort.col.value)
             stmt = stmt.order_by(sort_col.desc() if sort.dir is SortDir.desc else sort_col.asc())
+        if filter:
+            stmt = stmt.where(getattr(models.Dataset, filter.col.value).ilike(f"%{filter.value}%"))
         async with info.context.db() as session:
             datasets = await session.scalars(stmt)
         return connection_from_list(
@@ -293,100 +335,121 @@ class Query:
     async def compare_experiments(
         self,
         info: Info[Context, None],
-        experiment_ids: list[GlobalID],
+        baseline_experiment_id: GlobalID,
+        compare_experiment_ids: list[GlobalID],
+        first: Optional[int] = 50,
+        after: Optional[CursorString] = UNSET,
         filter_condition: Optional[str] = UNSET,
-    ) -> list[ExperimentComparison]:
-        experiment_ids_ = [
-            from_global_id_with_expected_type(experiment_id, OrmExperiment.__name__)
-            for experiment_id in experiment_ids
+    ) -> Connection[ExperimentComparison]:
+        if baseline_experiment_id in compare_experiment_ids:
+            raise BadRequest("Compare experiment IDs cannot contain the baseline experiment ID")
+        if len(set(compare_experiment_ids)) < len(compare_experiment_ids):
+            raise BadRequest("Compare experiment IDs must be unique")
+        experiment_ids = [
+            from_global_id_with_expected_type(experiment_id, models.Experiment.__name__)
+            for experiment_id in (baseline_experiment_id, *compare_experiment_ids)
         ]
-        if len(set(experiment_ids_)) != len(experiment_ids_):
-            raise ValueError("Experiment IDs must be unique.")
+        cursor = Cursor.from_string(after) if after else None
+        page_size = first or 50
 
         async with info.context.db() as session:
             validation_result = (
                 await session.execute(
                     select(
-                        func.count(distinct(OrmVersion.dataset_id)),
-                        func.max(OrmVersion.dataset_id),
-                        func.max(OrmVersion.id),
-                        func.count(OrmExperiment.id),
+                        func.count(distinct(models.DatasetVersion.dataset_id)),
+                        func.max(models.DatasetVersion.dataset_id),
+                        func.max(models.DatasetVersion.id),
+                        func.count(models.Experiment.id),
                     )
-                    .select_from(OrmVersion)
+                    .select_from(models.DatasetVersion)
                     .join(
-                        OrmExperiment,
-                        OrmExperiment.dataset_version_id == OrmVersion.id,
+                        models.Experiment,
+                        models.Experiment.dataset_version_id == models.DatasetVersion.id,
                     )
                     .where(
-                        OrmExperiment.id.in_(experiment_ids_),
+                        models.Experiment.id.in_(experiment_ids),
                     )
                 )
             ).first()
             if validation_result is None:
-                raise ValueError("No experiments could be found for input IDs.")
+                raise NotFound("No experiments could be found for input IDs.")
 
             num_datasets, dataset_id, version_id, num_resolved_experiment_ids = validation_result
             if num_datasets != 1:
-                raise ValueError("Experiments must belong to the same dataset.")
-            if num_resolved_experiment_ids != len(experiment_ids_):
-                raise ValueError("Unable to resolve one or more experiment IDs.")
+                raise BadRequest("Experiments must belong to the same dataset.")
+            if num_resolved_experiment_ids != len(experiment_ids):
+                raise NotFound("Unable to resolve one or more experiment IDs.")
 
             revision_ids = (
-                select(func.max(OrmRevision.id))
-                .join(OrmExample, OrmExample.id == OrmRevision.dataset_example_id)
+                select(func.max(models.DatasetExampleRevision.id))
+                .join(
+                    models.DatasetExample,
+                    models.DatasetExample.id == models.DatasetExampleRevision.dataset_example_id,
+                )
                 .where(
                     and_(
-                        OrmRevision.dataset_version_id <= version_id,
-                        OrmExample.dataset_id == dataset_id,
+                        models.DatasetExampleRevision.dataset_version_id <= version_id,
+                        models.DatasetExample.dataset_id == dataset_id,
                     )
                 )
-                .group_by(OrmRevision.dataset_example_id)
+                .group_by(models.DatasetExampleRevision.dataset_example_id)
                 .scalar_subquery()
             )
             examples_query = (
-                select(OrmExample)
-                .distinct(OrmExample.id)
+                select(models.DatasetExample)
+                .distinct(models.DatasetExample.id)
                 .join(
-                    OrmRevision,
+                    models.DatasetExampleRevision,
                     onclause=and_(
-                        OrmExample.id == OrmRevision.dataset_example_id,
-                        OrmRevision.id.in_(revision_ids),
-                        OrmRevision.revision_kind != "DELETE",
+                        models.DatasetExample.id
+                        == models.DatasetExampleRevision.dataset_example_id,
+                        models.DatasetExampleRevision.id.in_(revision_ids),
+                        models.DatasetExampleRevision.revision_kind != "DELETE",
                     ),
                 )
-                .order_by(OrmExample.id.desc())
+                .order_by(models.DatasetExample.id.desc())
+                .limit(page_size + 1)
             )
+            if cursor is not None:
+                examples_query = examples_query.where(models.DatasetExample.id < cursor.rowid)
 
             if filter_condition:
                 examples_query = update_examples_query_with_filter_condition(
                     query=examples_query,
                     filter_condition=filter_condition,
-                    experiment_ids=experiment_ids_,
+                    experiment_ids=experiment_ids,
                 )
 
             examples = (await session.scalars(examples_query)).all()
+            has_next_page = len(examples) > page_size
+            examples = examples[:page_size]
 
             ExampleID: TypeAlias = int
             ExperimentID: TypeAlias = int
-            runs: defaultdict[ExampleID, defaultdict[ExperimentID, list[OrmExperimentRun]]] = (
+            runs: defaultdict[ExampleID, defaultdict[ExperimentID, list[models.ExperimentRun]]] = (
                 defaultdict(lambda: defaultdict(list))
             )
             async for run in await session.stream_scalars(
-                select(OrmExperimentRun)
+                select(models.ExperimentRun)
                 .where(
                     and_(
-                        OrmExperimentRun.dataset_example_id.in_(example.id for example in examples),
-                        OrmExperimentRun.experiment_id.in_(experiment_ids_),
+                        models.ExperimentRun.dataset_example_id.in_(
+                            example.id for example in examples
+                        ),
+                        models.ExperimentRun.experiment_id.in_(experiment_ids),
                     )
                 )
-                .options(joinedload(OrmExperimentRun.trace).load_only(OrmTrace.trace_id))
+                .options(joinedload(models.ExperimentRun.trace).load_only(models.Trace.trace_id))
+                .order_by(
+                    models.ExperimentRun.repetition_number.asc()
+                )  # repetitions are not currently implemented, but this ensures that the repetitions will be properly ordered once implemented # noqa: E501
             ):
                 runs[run.dataset_example_id][run.experiment_id].append(run)
 
-        experiment_comparisons = []
+        cursors_and_nodes = []
         for example in examples:
             run_comparison_items = []
-            for experiment_id in experiment_ids_:
+            for experiment_id in experiment_ids:
                 run_comparison_items.append(
                     RunComparisonItem(
                         experiment_id=GlobalID(Experiment.__name__, str(experiment_id)),
@@ -398,17 +461,22 @@ class Query:
                         ],
                     )
                 )
-            experiment_comparisons.append(
-                ExperimentComparison(
-                    example=DatasetExample(
-                        id_attr=example.id,
-                        created_at=example.created_at,
-                        version_id=version_id,
-                    ),
-                    run_comparison_items=run_comparison_items,
-                )
+            experiment_comparison = ExperimentComparison(
+                id_attr=example.id,
+                example=DatasetExample(
+                    id_attr=example.id,
+                    created_at=example.created_at,
+                    version_id=version_id,
+                ),
+                run_comparison_items=run_comparison_items,
             )
-        return experiment_comparisons
+            cursors_and_nodes.append((Cursor(rowid=example.id), experiment_comparison))
+
+        return connection_from_cursors_and_nodes(
+            cursors_and_nodes=cursors_and_nodes,
+            has_previous_page=False,  # set to false since we are only doing forward pagination (https://relay.dev/graphql/connections.htm#sec-undefined.PageInfo.Fields) # noqa: E501
+            has_next_page=has_next_page,
+        )
 
     @strawberry.field
     async def validate_experiment_run_filter_condition(
@@ -420,7 +488,7 @@ class Query:
             compile_sqlalchemy_filter_condition(
                 filter_condition=condition,
                 experiment_ids=[
-                    from_global_id_with_expected_type(experiment_id, OrmExperiment.__name__)
+                    from_global_id_with_expected_type(experiment_id, models.Experiment.__name__)
                     for experiment_id in experiment_ids
                 ],
             )
@@ -437,16 +505,13 @@ class Query:
     @strawberry.field
     async def functionality(self, info: Info[Context, None]) -> "Functionality":
         has_model_inferences = not info.context.model.is_empty
-        async with info.context.db() as session:
-            has_traces = (await session.scalar(select(models.Trace).limit(1))) is not None
         return Functionality(
             model_inferences=has_model_inferences,
-            tracing=has_traces,
         )
 
     @strawberry.field
-    def model(self) -> Model:
-        return Model()
+    def model(self) -> InferenceModel:
+        return InferenceModel()
 
     @strawberry.field
     async def node(self, id: GlobalID, info: Info[Context, None]) -> Node:
@@ -596,6 +661,43 @@ class Query:
                 ):
                     raise NotFound(f"Unknown prompt label: {id}")
             return to_gql_prompt_label(prompt_label)
+        elif type_name == PromptVersionTag.__name__:
+            async with info.context.db() as session:
+                if not (prompt_version_tag := await session.get(models.PromptVersionTag, node_id)):
+                    raise NotFound(f"Unknown prompt version tag: {id}")
+            return to_gql_prompt_version_tag(prompt_version_tag)
+        elif type_name == ProjectTraceRetentionPolicy.__name__:
+            async with info.context.db() as session:
+                db_policy = await session.scalar(
+                    select(models.ProjectTraceRetentionPolicy).filter_by(id=node_id)
+                )
+                if not db_policy:
+                    raise NotFound(f"Unknown project trace retention policy: {id}")
+            return ProjectTraceRetentionPolicy(id=db_policy.id, db_policy=db_policy)
+        elif type_name == SpanAnnotation.__name__:
+            async with info.context.db() as session:
+                span_annotation = await session.get(models.SpanAnnotation, node_id)
+                if not span_annotation:
+                    raise NotFound(f"Unknown span annotation: {id}")
+            return to_gql_span_annotation(span_annotation)
+        elif type_name == TraceAnnotation.__name__:
+            async with info.context.db() as session:
+                trace_annotation = await session.get(models.TraceAnnotation, node_id)
+                if not trace_annotation:
+                    raise NotFound(f"Unknown trace annotation: {id}")
+            return to_gql_trace_annotation(trace_annotation)
+        elif type_name == GenerativeModel.__name__:
+            async with info.context.db() as session:
+                stmt = (
+                    select(models.GenerativeModel)
+                    .where(models.GenerativeModel.deleted_at.is_(None))
+                    .where(models.GenerativeModel.id == node_id)
+                    .options(joinedload(models.GenerativeModel.token_prices))
+                )
+                model = await session.scalar(stmt)
+                if not model:
+                    raise NotFound(f"Unknown model: {id}")
+            return to_gql_generative_model(model)
         raise NotFound(f"Unknown node type: {type_name}")
 
     @strawberry.field
@@ -665,6 +767,28 @@ class Query:
                 args=args,
             )
 
+    @strawberry.field
+    async def annotation_configs(
+        self,
+        info: Info[Context, None],
+        first: Optional[int] = 50,
+        last: Optional[int] = None,
+        after: Optional[str] = None,
+        before: Optional[str] = None,
+    ) -> Connection[AnnotationConfig]:
+        args = ConnectionArgs(
+            first=first,
+            after=after if isinstance(after, CursorString) else None,
+            last=last,
+            before=before if isinstance(before, CursorString) else None,
+        )
+        async with info.context.db() as session:
+            configs = await session.stream_scalars(
+                select(models.AnnotationConfig).order_by(models.AnnotationConfig.name)
+            )
+            data = [to_gql_annotation_config(config) async for config in configs]
+            return connection_from_list(data=data, args=args)
+
     @strawberry.field
     def clusters(
         self,
@@ -793,6 +917,45 @@ class Query:
             clustered_events=clustered_events,
         )
 
+    @strawberry.field
+    async def default_project_trace_retention_policy(
+        self,
+        info: Info[Context, None],
+    ) -> ProjectTraceRetentionPolicy:
+        stmt = select(models.ProjectTraceRetentionPolicy).filter_by(
+            id=DEFAULT_PROJECT_TRACE_RETENTION_POLICY_ID
+        )
+        async with info.context.db() as session:
+            db_policy = await session.scalar(stmt)
+        assert db_policy
+        return ProjectTraceRetentionPolicy(id=db_policy.id, db_policy=db_policy)
+
+    @strawberry.field
+    async def project_trace_retention_policies(
+        self,
+        info: Info[Context, None],
+        first: Optional[int] = 100,
+        last: Optional[int] = UNSET,
+        after: Optional[CursorString] = UNSET,
+        before: Optional[CursorString] = UNSET,
+    ) -> Connection[ProjectTraceRetentionPolicy]:
+        args = ConnectionArgs(
+            first=first,
+            after=after if isinstance(after, CursorString) else None,
+            last=last,
+            before=before if isinstance(before, CursorString) else None,
+        )
+        stmt = select(models.ProjectTraceRetentionPolicy).order_by(
+            models.ProjectTraceRetentionPolicy.id
+        )
+        async with info.context.db() as session:
+            result = await session.stream_scalars(stmt)
+            data = [
+                ProjectTraceRetentionPolicy(id=db_policy.id, db_policy=db_policy)
+                async for db_policy in result
+            ]
+        return connection_from_list(data=data, args=args)
+
     @strawberry.field(
         description="The allocated storage capacity of the database in bytes. "
         "Return None if this information is unavailable.",
@@ -821,13 +984,14 @@ class Query:
             #     stats = cast(Iterable[tuple[str, int]], await session.execute(stmt))
             # stats = _consolidate_sqlite_db_table_stats(stats)
         elif info.context.db.dialect is SupportedSQLDialect.POSTGRESQL:
-            stmt = text(f"""\
+            nspname = getenv(ENV_PHOENIX_SQL_DATABASE_SCHEMA) or "public"
+            stmt = text("""\
                 SELECT c.relname, pg_total_relation_size(c.oid)
                 FROM pg_class as c
                 INNER JOIN pg_namespace as n ON n.oid = c.relnamespace
                 WHERE c.relkind = 'r'
-                AND n.nspname = '{getenv(ENV_PHOENIX_SQL_DATABASE_SCHEMA) or "public"}';
-            """)
+                AND n.nspname = :nspname;
+            """).bindparams(nspname=nspname)
             try:
                 async with info.context.db() as session:
                     stats = cast(Iterable[tuple[str, int]], await session.execute(stmt))
@@ -841,6 +1005,53 @@ class Query:
             for table_name, num_bytes in stats
         ]
 
+    @strawberry.field
+    def validate_regular_expression(self, regex: str) -> ValidationResult:
+        try:
+            re.compile(regex)
+            return ValidationResult(is_valid=True, error_message=None)
+        except re.error as error:
+            return ValidationResult(is_valid=False, error_message=str(error))
+
+    @strawberry.field
+    async def get_span_by_otel_id(
+        self,
+        info: Info[Context, None],
+        span_id: str,
+    ) -> Optional[Span]:
+        stmt = select(models.Span.id).filter_by(span_id=span_id)
+        async with info.context.db() as session:
+            span_rowid = await session.scalar(stmt)
+        if span_rowid:
+            return Span(span_rowid=span_rowid)
+        return None
+
+    @strawberry.field
+    async def get_trace_by_otel_id(
+        self,
+        info: Info[Context, None],
+        trace_id: str,
+    ) -> Optional[Trace]:
+        stmt = select(models.Trace.id).where(models.Trace.trace_id == trace_id)
+        async with info.context.db() as session:
+            trace_rowid = await session.scalar(stmt)
+        if trace_rowid:
+            return Trace(trace_rowid=trace_rowid)
+        return None
+
+    @strawberry.field
+    async def get_project_session_by_otel_id(
+        self,
+        info: Info[Context, None],
+        session_id: str,
+    ) -> Optional[ProjectSession]:
+        stmt = select(models.ProjectSession).where(models.ProjectSession.session_id == session_id)
+        async with info.context.db() as session:
+            session_row = await session.scalar(stmt)
+        if session_row:
+            return to_gql_project_session(session_row)
+        return None
+
 
 def _consolidate_sqlite_db_table_stats(
     stats: Iterable[tuple[str, int]],
